import numpy as np
import matplotlib
matplotlib.use('Agg')
import matplotlib.pyplot as plt

plot_range = [0., 4.5, -1.1, 1.1]
ds = 0.4  #step size of branch following through continuation
s_max = 3.5  #length of branch following through continuation
N = int( round(s_max/ds) )
prefix = ""

################################################

def plot_bif_diagr1(bif_diagr, error_bars, N, plot_range, prefix):
	plt.plot(bif_diagr[1],bif_diagr[0], color=(0.0,0.165,0.424), linestyle='-', marker='o', markerfacecolor=(0.0,0.165,0.424))
	for i in range(N+1):
		plot_error_bar(i, bif_diagr, error_bars, plot_range)
	#plt.savefig("{}bifurcation_diagram.png".format(prefix))
	#plt.show()

def plot_bif_diagr2(bif_diagr, error_bars, N, plot_range, prefix):
	plt.plot(bif_diagr[1],bif_diagr[0], color=(0.333, 0.420, 0.184), linestyle='-', marker='D', markerfacecolor=(0.333, 0.420, 0.184))
	for i in range(N+1):
		plot_error_bar(i, bif_diagr, error_bars, plot_range)
	#plt.savefig("{}bifurcation_diagram.png".format(prefix))
	#plt.show()

################################################

#error bars in bifurcation diagram:
def plot_error_bar(i, bif_diagr, error_bars, plot_range):
	if error_bars[0,i] == 0.:
		return
	p0 = bif_diagr[:,i]
	p1 = p0 + error_bars[0,i] * error_bars[1:,i]
	p2 = p0 - error_bars[0,i] * error_bars[1:,i]
	dx = np.array([error_bars[2,i],-error_bars[1,i]])
	size_x = (plot_range[1]-plot_range[0])/100.
	size_h = (plot_range[3]-plot_range[2])/100.
	size = size_h * np.absolute(dx[0]) + size_x * np.absolute(dx[1])
	p3 = p2 + size * dx #0.01
	p4 = p2 - size * dx
	p5 = p1 + size * dx
	p6 = p1 - size * dx
	plt.plot([p1[1], p2[1]], [p1[0], p2[0]], color=(0.62,0.1,0.11), linewidth='1.5')
	plt.plot([p3[1], p4[1]], [p3[0], p4[0]], color=(0.62,0.1,0.11), linewidth='1.5')
	plt.plot([p5[1], p6[1]], [p5[0], p6[0]], color=(0.62,0.1,0.11), linewidth='1.5')

################################################

bd1 = np.load("bifurcation_diagram_1stOrderStructureLifting.npy")
bd2 = np.load("bifurcation_diagram_2ndOrderStructureLifting.npy")
eb1 = np.load("error_bars_1stOrderStructureLifting.npy")
eb2 = np.load("error_bars_2ndOrderStructureLifting.npy")

plt.figure("{}bifurcation diagram".format(prefix))#, figsize=(24, 12))
plt.rc('xtick',labelsize=12)
plt.rc('ytick',labelsize=12)
plt.axis(plot_range)
#plt.grid()
plt.xlabel(r'$kT/J$',fontsize=18)
plt.ylabel(r'$m$',fontsize=18)

#exact solution:
t = np.linspace(0.01, 2./np.log(1+np.sqrt(2.)), 200)
tt = np.linspace(0.01, 4.5, 200)
plt.plot(t,(1-np.sinh(2./t)**(-4))**(1/8.), color=(0.0,0.165,0.424), linestyle='--', linewidth=2)
plt.plot(t,-(1-np.sinh(2./t)**(-4))**(1/8.), color=(0.0,0.165,0.424), linestyle='--', linewidth=2)
plt.plot(tt, np.zeros(200), color=(0.0,0.165,0.424), linestyle='--', linewidth=2)

#stochastic continuation results:
plot_bif_diagr1(bd1, eb1, N, plot_range, prefix)
plot_bif_diagr2(bd2, eb2, N, plot_range, prefix)

plt.savefig("{}bifurcation_diagram.png".format(prefix), bbox_inches='tight')
